
#ifndef __GABEDIT_RAMANSPECTRUM_H__
#define __GABEDIT_RAMANSPECTRUM_H__

void createRamanSpectrum(GtkWidget *parentWindow, GabEditTypeFile typeOfFile);
void createRamanSpectrumFromVibration(GtkWidget *parentWindow, Vibration ibration);

#endif /* __GABEDIT_RAMANSPECTRUM_H__ */
