
#ifndef __GABEDIT_PSICODEGLOBAL_H__
#define __GABEDIT_PSICODEGLOBAL_H__

extern GtkWidget* psicodeWin;
extern PsicodeMolecule psicodeMolecule;
extern PsicodeColorFore psicodeColorFore;
extern PsicodeColorBack psicodeColorBack;
extern PsicodeBasis psicodeBasis;
extern PsicodeGuessWaveFunction psicodeGuessWaveFunction;

extern PsicodeFunctional* functionalsPsi;
extern PsicodeStdFunctional* stdFunctionalsPsi;
extern gdouble* sumFunctionalsPsi;

#endif /* __GABEDIT_PSICODEGLOBAL_H__ */

