
#ifndef __GABEDIT_MOLCASGLOBAL_H__
#define __GABEDIT_MOLCASGLOBAL_H__

extern GtkWidget* molcasWin;
extern MolcasMolecule molcasMolecule;
extern MolcasOptimization molcasOptimization;
extern MolcasSolvation molcasSolvation;
extern MolcasScf molcasScf;
extern MolcasGridIT molcasGridIT;
extern MolcasColorFore molcasColorFore;
extern MolcasColorBack molcasColorBack;
extern MolcasSystemVariables molcasSystemVariables;

#endif /* __GABEDIT_MOLCASGLOBAL_H__ */

