
#ifndef __GABEDIT_CONSTANTS_H__
#define __GABEDIT_CONSTANTS_H__

#define BSIZE 1024
#define ANG_TO_BOHR  1.0/0.52917726
#define BOHR_TO_ANG  0.52917726
#define DEG_TO_RAD   0.017453293 
#define AUTODEB  2.54158059
#define AUTOEV  27.21138469
#define PI   3.14159265358979323846
#define DEG_TO_RAD   0.017453293 
#define RAD_TO_DEG   57.29577951308232090712
#define AMU_TO_AU 1822.88848121
#define AU_TO_CM1 219474.63633664 
#define slight (299792458.0) 
#define NAvogadro (6.02214129e23)
#define hPlank (6.6260693e-34)
#define epsilon0 (8.854187817e-12)
#define AUTOKCAL 627.509544796
#define KCALTOKJ 4.18400000


#endif /* __GABEDIT_CONSTANTS_H__ */
