#ifndef __GABEDIT_FIREFLYSCF_H__
#define __GABEDIT_FIREFLYSCF_H__

void initFireFlySCFFrame();
void setSensitiveFireFlySCFFrame(gboolean sensitive);
void putFireFlySCFInfoInTextEditor();
void createFireFlySCFFrame(GtkWidget *win, GtkWidget *box);

#endif /* __GABEDIT_FIREFLYSCF_H__ */
