
#ifndef __GABEDIT_DEMONGLOBAL_H__
#define __GABEDIT_DEMONGLOBAL_H__

extern GtkWidget* demonWin;
extern DeMonMolecule demonMolecule;
extern DeMonColorFore demonColorFore;
extern DeMonColorBack demonColorBack;
extern DeMonBasis demonBasis;
extern DeMonGuessWaveFunction demonGuessWaveFunction;

extern DeMonFunctional* functionalsDeMon;
extern DeMonStdFunctional* stdFunctionalsDeMon;
extern gdouble* sumFunctionalsDeMon;
#endif /* __GABEDIT_DEMONGLOBAL_H__ */

