
#ifndef __GABEDIT_GAMESSGLOBAL_H__
#define __GABEDIT_GAMESSGLOBAL_H__

extern GtkWidget* gamessWin;
extern GamessMolecule gamessMolecule;
extern GamessColorFore gamessColorFore;
extern GamessColorBack gamessColorBack;
extern GamessBasis gamessBasis;

extern GamessFunctional* functionalsGamess;
extern GamessStdFunctional* stdFunctionalsGamess;
extern gdouble* sumFunctionalsGamess;


#endif /* __GABEDIT_GAMESSGLOBAL_H__ */

