
#ifndef __GABEDIT_ORCAGLOBAL_H__
#define __GABEDIT_ORCAGLOBAL_H__

extern GtkWidget* orcaWin;
extern OrcaMolecule orcaMolecule;
extern OrcaColorFore orcaColorFore;
extern OrcaColorBack orcaColorBack;
extern OrcaBasis orcaBasis;
extern OrcaGuessWaveFunction orcaGuessWaveFunction;

extern OrcaFunctional* functionalsOrca;
extern OrcaStdFunctional* stdFunctionalsOrca;
extern gdouble* sumFunctionalsOrca;

#endif /* __GABEDIT_ORCAGLOBAL_H__ */

