
#ifndef __GABEDIT_MOPACGLOBAL_H__
#define __GABEDIT_MOPACGLOBAL_H__

extern GtkWidget* mopacWin;
extern MopacMolecule mopacMolecule;
extern MopacColorFore mopacColorFore;
extern MopacColorBack mopacColorBack;

extern MopacFunctional* functionalsMopac;
extern MopacStdFunctional* stdFunctionalsMopac;
extern gdouble* sumFunctionalsMopac;

#endif /* __GABEDIT_MOPACGLOBAL_H__ */

