
#ifndef __GABEDIT_NWCHEMGLOBAL_H__
#define __GABEDIT_NWCHEMGLOBAL_H__

extern GtkWidget* nwchemWin;
extern NWChemMolecule nwchemMolecule;
extern NWChemColorFore nwchemColorFore;
extern NWChemColorBack nwchemColorBack;
extern NWChemBasis nwchemBasis;
extern NWChemGuessWaveFunction nwchemGuessWaveFunction;

extern NWChemFunctional* functionalsNWC;
extern NWChemStdFunctional* stdFunctionalsNWC;
extern gdouble* sumFunctionalsNWC;

#endif /* __GABEDIT_NWCHEMGLOBAL_H__ */

