
#ifndef __GABEDIT_MPQCGLOBAL_H__
#define __GABEDIT_MPQCGLOBAL_H__

extern GtkWidget* mpqcWin;
extern MPQCMolecule mpqcMolecule;
extern MPQCColorFore mpqcColorFore;
extern MPQCColorBack mpqcColorBack;
extern MPQCMole mpqcMole;
extern MPQCGuessWaveFunction mpqcGuessWaveFunction;
extern MPQCMpqc mpqcMpqc;
extern MPQCBasis mpqcBasis;

extern MPQCFunctional* functionalsMPQC;
extern MPQCStdFunctional* stdFunctionalsMPQC;
extern gdouble* sumFunctionalsMPQC;
extern MPQCOptimization mpqcOptimization;


#endif /* __GABEDIT_MPQCGLOBAL_H__ */

