
#ifndef __GABEDIT_FIREFLYGLOBAL_H__
#define __GABEDIT_FIREFLYGLOBAL_H__

extern GtkWidget* fireflyWin;
extern FireFlyMolecule fireflyMolecule;
extern FireFlyColorFore fireflyColorFore;
extern FireFlyColorBack fireflyColorBack;
extern FireFlyBasis fireflyBasis;

extern FireFlyFunctional* functionalsFF;
extern FireFlyStdFunctional* stdFunctionalsFF;
extern gdouble* sumFunctionalsFF;

#endif /* __GABEDIT_FIREFLYGLOBAL_H__ */

