
#ifndef __GABEDIT_STOCKICONS_H__
#define __GABEDIT_STOCKICONS_H__

#define GABEDIT_STOCK_A0D    		"gabedit-a0d"
#define GABEDIT_STOCK_A0P  		"gabedit-a0p"
#define GABEDIT_STOCK_ABOUT  		"gabedit-about"
#define GABEDIT_STOCK_APPLY  		"gabedit-apply"
#define GABEDIT_STOCK_BOOK_CLOSE  	"gabedit-book-close"
#define GABEDIT_STOCK_BOOK_OPEN  	"gabedit-book-open"
#define GABEDIT_STOCK_BOOK_PARENT  	"gabedit-book-parent"
#define GABEDIT_STOCK_CAMERA  		"gabedit-book-camera"
#define GABEDIT_STOCK_CANCEL  		"gabedit-cancel"
#define GABEDIT_STOCK_CLOSE  		"gabedit-close"
#define GABEDIT_STOCK_COPY  		"gabedit-copy"
#define GABEDIT_STOCK_CUT  		"gabedit-cut"
#define GABEDIT_STOCK_DELETE_ATOM 	"gabedit-delete-atom"
#define GABEDIT_STOCK_DRAW 		"gabedit-draw"
#define GABEDIT_STOCK_IFRAG 		"gabedit-ifrag"
#define GABEDIT_STOCK_DRIVER 		"gabedit-driver"
#define GABEDIT_STOCK_ERROR 		"gabedit-error"
#define GABEDIT_STOCK_EXIT 		"gabedit-exit"
#define GABEDIT_STOCK_FIND 		"gabedit-find"
#define GABEDIT_STOCK_GAUSSIAN		"gabedit-gaussian"
#define GABEDIT_STOCK_GEOMETRY		"gabedit-geometry"
#define GABEDIT_STOCK_HELP		"gabedit-help"
#define GABEDIT_STOCK_HIDE		"gabedit-hide"
#define GABEDIT_STOCK_ADJUST_H		"gabedit-adjust-h"
#define GABEDIT_STOCK_HOME		"gabedit-home"
#define GABEDIT_STOCK_H			"gabedit-h"
#define GABEDIT_STOCK_GABEDIT		"gabedit-gabedit"
#define GABEDIT_STOCK_INFO		"gabedit-info"
#define GABEDIT_STOCK_INSERT_ATOM	"gabedit-insert-atom"
#define GABEDIT_STOCK_INSERT		"gabedit-insert"
#define GABEDIT_STOCK_CHANGE_BOND	"gabedit-change-bond"
#define GABEDIT_STOCK_CUT_BOND		"gabedit-cut-bond"
#define GABEDIT_STOCK_DELETE_CUT	"gabedit-delete-cut"
#define GABEDIT_STOCK_ADD_BOND		"gabedit-add-bond"
#define GABEDIT_STOCK_MEASURE		"gabedit-measure"
#define GABEDIT_STOCK_MOLCAS		"gabedit-molcas"
#define GABEDIT_STOCK_MOLDEN		"gabedit-molden"
#define GABEDIT_STOCK_MOLPRO		"gabedit-molpro"
#define GABEDIT_STOCK_QCHEM		"gabedit-qchem"
#define GABEDIT_STOCK_ORCA		"gabedit-orca"
#define GABEDIT_STOCK_VASP		"gabedit-vasp"
#define GABEDIT_STOCK_MOPAC		"gabedit-mopac"
#define GABEDIT_STOCK_ADF		"gabedit-adf"
#define GABEDIT_STOCK_DALTON		"gabedit-dalton"
#define GABEDIT_STOCK_GAMESS		"gabedit-gamess"
#define GABEDIT_STOCK_DEMON		"gabedit-demon"
#define GABEDIT_STOCK_NWCHEM		"gabedit-nwchem"
#define GABEDIT_STOCK_PSICODE		"gabedit-psicode"
#define GABEDIT_STOCK_NBO		"gabedit-nbo"
#define GABEDIT_STOCK_FIREFLY		"gabedit-firefly"
#define GABEDIT_STOCK_MOVE_ATOM		"gabedit-move-atom"
#define GABEDIT_STOCK_MPQC		"gabedit-mpqc"
#define GABEDIT_STOCK_NEW		"gabedit-new"
#define GABEDIT_STOCK_NEXTW		"gabedit-next"
#define GABEDIT_STOCK_NO		"gabedit-no"
#define GABEDIT_STOCK_OK		"gabedit-ok"
#define GABEDIT_STOCK_OPEN_BABEL	"gabedit-open-babel"
#define GABEDIT_STOCK_CONVERT_UNIT	"gabedit-convert-unit"
#define GABEDIT_STOCK_OPEN		"gabedit-open"
#define GABEDIT_STOCK_ORBITALS		"gabedit-orbitals"
#define GABEDIT_STOCK_ORTHO		"gabedit-ortho"
#define GABEDIT_STOCK_PAGE		"gabedit-page"
#define GABEDIT_STOCK_PASTE		"gabedit-paste"
#define GABEDIT_STOCK_PERSPECTIVE	"gabedit-perspective"
#define GABEDIT_STOCK_PLAY		"gabedit-play"
#define GABEDIT_STOCK_PRINT		"gabedit-print"
#define GABEDIT_STOCK_QUESTION		"gabedit-question"
#define GABEDIT_STOCK_RENDER_BALL_STICK	"gabedit-render-ball-stick"
#define GABEDIT_STOCK_RENDER_STICK	"gabedit-render-stick"
#define GABEDIT_STOCK_RENDER_WIREFRAME	"gabedit-render-wireframe"
#define GABEDIT_STOCK_ROTATION_LOCAL	"gabedit-rotate-local"
#define GABEDIT_STOCK_ROTATION		"gabedit-rotate"
#define GABEDIT_STOCK_ROTATION_X		"gabedit-rotate-x"
#define GABEDIT_STOCK_ROTATION_Y		"gabedit-rotate-y"
#define GABEDIT_STOCK_ROTATION_Z		"gabedit-rotate-z"
#define GABEDIT_STOCK_ROTATION_Z_LOCAL	"gabedit-rotate-z-local"
#define GABEDIT_STOCK_RUN		"gabedit-run"
#define GABEDIT_STOCK_SAVE_AS		"gabedit-save-as"
#define GABEDIT_STOCK_SELECT_ALL	"gabedit-select-all"
#define GABEDIT_STOCK_SAVE_MINI		"gabedit-save-mini"
#define GABEDIT_STOCK_SAVE		"gabedit-save"
#define GABEDIT_STOCK_SCALE_BALL	"gabedit-scale-ball"
#define GABEDIT_STOCK_SCALE_DIPOLE	"gabedit-scale-dipole"
#define GABEDIT_STOCK_SCALE_STICK	"gabedit-scale-stick"
#define GABEDIT_STOCK_SELECT_RESIDUE	"gabedit-select-residue"
#define GABEDIT_STOCK_SELECT		"gabedit-select"
#define GABEDIT_STOCK_STOP		"gabedit-stop"
#define GABEDIT_STOCK_TRANSLATE		"gabedit-translate"
#define GABEDIT_STOCK_WARNING		"gabedit-warning"
#define GABEDIT_STOCK_YES		"gabedit-yes"
#define GABEDIT_STOCK_ZOOM		"gabedit-zoom"
#define GABEDIT_STOCK_PDB		"gabedit-pdb"
#define GABEDIT_STOCK_O			"gabedit-o"
#define GABEDIT_STOCK_ATOMTOINSERT	"gabedit-atomtoinsert"
#define GABEDIT_STOCK_KNOT		"gabedit-knot"

void gabedit_gtk_stock_init();

#endif /* __GABEDIT_STOCKICONS_H__ */

