
#ifndef __GABEDIT_QCHEMGLOBAL_H__
#define __GABEDIT_QCHEMGLOBAL_H__

extern GtkWidget* qchemWin;
extern QChemMolecule qchemMolecule;
extern QChemColorFore qchemColorFore;
extern QChemColorBack qchemColorBack;
extern QChemBasis qchemBasis;
extern QChemGuessWaveFunction qchemGuessWaveFunction;

extern QChemFunctional* functionalsQC;
extern QChemStdFunctional* stdFunctionalsQC;
extern gdouble* sumFunctionalsQC;
#endif /* __GABEDIT_QCHEMGLOBAL_H__ */

